import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.apache.xerces.parsers.DOMParser;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/*****************************************************
 * 
 * @author amanda
 *
 * Annotation Model is a generic class 
 * Specific annotation models derive from this class
 */
 public class AnnotationModel {
    protected Document doc;
    protected Document schema;
    protected File outfile = null;
    protected File infile = null;
    protected File annotationfile = null;
    
    //for the annotation comparison
    protected File outfile2 = null;
    protected File infile2 = null; 
    protected Document doc2;

    public AnnotationModel() { }
    
    public Document readDocument(File filename) {
	try {
	    DOMParser parser = new DOMParser();
	    parser.parse(new InputSource(new FileReader(filename)));
	    Document d = parser.getDocument();
	    return d;
	} catch (Exception ex) {
	    return null;
	}
    }

    protected Vector getAnnotationLabels(int index) {
	Node node = schema.getElementsByTagName("category").item(index);
	NodeList nodes = schema.getElementsByTagName("label");
	Vector results = new Vector();
	for(int i = 0; i < nodes.getLength(); i++) {
	    if(nodes.item(i).getParentNode() == node) {
		if(nodes.item(i).getAttributes().getNamedItem("type") != null) {
		    results.add(new LabelPair(nodes.item(i).getFirstChild().getNodeValue(), nodes.item(i).getAttributes().getNamedItem("type").getNodeValue()));
		} else {
		    results.add(new LabelPair(nodes.item(i).getFirstChild().getNodeValue(), ""));
		}
	    }
	}
	return results;	    
    }

    private void printToFile(Document thedoc, File theoutfile) {
	try {
	    OutputFormat format = new OutputFormat(thedoc);
	    format.setIndenting(true);
	    
	    XMLSerializer serializer = new XMLSerializer(new FileOutputStream(theoutfile), format);
	    serializer.serialize(thedoc);
	    
	} catch(IOException ie) {
	    ie.printStackTrace();
	}
    }

    /*****************************************************
     * 
     * @param e
     * @param attribute  name of an attribute
     * @param value      can be a Labeled Pair or a String
     */
    public void setAnnotation(Element e, String attribute, Object value) {
    	
    if(value ==null) return;
	if(!value.toString().equalsIgnoreCase("none")) {
	    e.setAttribute(attribute, value.toString());
	    if( value.getClass().toString().equals("class LabelPair") && 
	    		((LabelPair)value).type.equals("comment")) {	
		String c = getComment();
		e.setAttribute(attribute+"_comment", c);
	    }
	} else {
	    e.removeAttribute(attribute);
	    e.removeAttribute(attribute+"_comment");	    
	}
	printToFile(doc, outfile);
	//printToFile(doc2, outfile2);
    }

    protected String getComment() {
	String s = (String)JOptionPane.showInputDialog("");
	return s;
    }    
    
    /*************************************
     * generic methods. to be overwritten by subclasses
     */
    protected NodeList getUtterances(){
    	return doc.getElementsByTagName("utterance");
    	}
 
    //for comparison- second file
    protected NodeList getUtterances2(){
    	return null;
    	}
    
    //pure abstract methods 
    protected Vector getAttColumnNames(Vector columnNames){		
	   NamedNodeMap docNodes = getUtterances().item(0).getAttributes();
		
	    //add attribute as columns to the table
	    for(int i = 0; i < docNodes.getLength(); i++) {
		columnNames.add(new NamePair(docNodes.item(i).getNodeName(), 50, false));
	    }
	    columnNames.add(new NamePair("Utterance", 1000, false));
	    return columnNames;
	    };
	    
	/***********************************************
	 *  
	 * @param node
	 * @return text to display in the Utterance column
	 * default version returns text of the first child
	 */
	protected Object getUtteranceText(Node node)
	{
		return node.getFirstChild().getNodeValue();
	}
	
	/************************************************
	 * 
	 * @param colName - column name
	 * @param thisnode - xml node to retrieve value from
	 * @return if the column name is "Utterance" return utterance text, else 
	 * 			return the the attribute value using col name
	 */
	protected Object getValueAt(String colName, Node thisnode)
	{
		if(colName.equals("Utterance"))
		{
			return getUtteranceText(thisnode);
		}
		else
		{
		    try {
		    	return thisnode.getAttributes().getNamedItem(colName).getNodeValue();
		    } catch(Exception e) {
		    	//attribute for this column name does not exist
		    	return "";
			} 
			
		}
 	}
	
	/**********************************************
	 * Output caption
	 */
	public String toString()
	{
		String caption = "";
		if(annotationfile!=null)
		{
			String str = annotationfile.toString();
			if (str.lastIndexOf('\\') > 0)
			{
				str = str.substring(str.lastIndexOf('\\'));
			}
			caption += " Scheme: "+str;
		}
		if(infile!=null)
		{
			String str = infile.toString();
			caption += " FileIN: "+str.substring(str.lastIndexOf('\\'));
		}
		if(infile2!=null)
		{
			String str = infile2.toString();
			caption += " FileIN2: "+str.substring(str.lastIndexOf('\\'));
		}

	return caption;
	}
    
}