import java.util.Vector;
import java.util.Map;
import java.util.*;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.FileReader;

import javax.swing.AbstractListModel;
import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.ListCellRenderer;

import javax.swing.filechooser.FileFilter;

import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableCellRenderer;

import javax.swing.event.TableModelEvent;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.xml.sax.InputSource;

import org.apache.xerces.parsers.DOMParser;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;



/** 
 * @author Amanda Stent.
 * 
 * 
 * AnnotationnPanel class implements the user interface functionality
 * of the annotation program
 * 
 * Supports representation in the form of table and TODO: list 
 **/
public class AnnotationPanel extends JPanel implements ActionListener {
    private AnnotationModel am;
    private JTable table;
    private JList list;
    private JFileChooser fc;
    private static JFrame frame;
    private static String model_type ="";
    private static String model_file ="";
    private static String model_compare ="";
    
    public AnnotationPanel() {
        super(new BorderLayout());

        if(model_type.equals("monroe"))
        	am = new MonroeAnnotationModel();
        else
        if(model_type.equals("comm"))
            am = new CommAnnotationModel();
        else
            if(model_type.equals("toy"))
                am = new ToyAnnotationModel();
        else           	
        	am = new AnnotationModel();
        	
		if(!model_file.equals("") )
		{
			File file = new File(model_file);
			am.annotationfile = file;    
		}
    
		if(model_compare.equals("compare"))
		//load comparison model
			table = new JTable(new CompareTableModel(am));
		else	
		//load regular model
			table = new JTable(new UtteranceTableModel(am));

	list = new JList(new MyListModel(am));
	fc = new JFileChooser();
	fc.setFileFilter(new XMLFilter());
	fc.setAcceptAllFileFilterUsed(false);
    }

    public void setUpTable() {
        table.setPreferredScrollableViewportSize(new Dimension(1000, 200));
	removeAll();
        JScrollPane scrollPane = new JScrollPane(table);
        add(scrollPane, BorderLayout.CENTER);
	repaint();
    }

    public void setUpList() {
	removeAll();
        JScrollPane scrollPane = new JScrollPane(list);
        add(scrollPane, BorderLayout.CENTER);
	add(((MyListModel)list.getModel()).jp, BorderLayout.SOUTH);
	repaint();
    }
	
    public void actionPerformed(ActionEvent e) {
	String sourceText = e.getActionCommand();
	UtteranceTableModel mtm = (UtteranceTableModel)table.getModel();
	MyListModel ml = (MyListModel)list.getModel();
	AnnotationModel am = mtm.am;

	if(sourceText.equals("Open File")) {
	    int returnVal = fc.showOpenDialog(table);
	    if (returnVal == JFileChooser.APPROVE_OPTION) {
		File file = fc.getSelectedFile();
		fc.setCurrentDirectory(file);
		am.infile = file;
		mtm.setUpTable(table);
		ml.setUpList(list);

		String fn = file.toString();
		am.outfile = new File(fn.substring(0, fn.lastIndexOf(".")) + "_out" + fn.substring(fn.lastIndexOf("."), fn.length()));
		
	    }

	} else 	if(sourceText.equals("Open Second File")) {
	    int returnVal = fc.showOpenDialog(table);
	    if (returnVal == JFileChooser.APPROVE_OPTION) {
		File file = fc.getSelectedFile();
		fc.setCurrentDirectory(file);
		am.infile2 = file;
		mtm.setUpTable(table);
		ml.setUpList(list);
		if(am.outfile2 == null) {
		    String fn = file.toString();
		    am.outfile2 = new File(fn.substring(0, fn.lastIndexOf(".")) + "_out" + fn.substring(fn.lastIndexOf("."), fn.length()));
		}
	    }

	} else if(sourceText.equals("Save As")) {
	    int returnVal = fc.showSaveDialog(table);
	    if (returnVal == JFileChooser.APPROVE_OPTION) {
		File file = fc.getSelectedFile();
		fc.setCurrentDirectory(file);
		am.outfile = file;
	    }

	} else if(sourceText.equals("Open Annotation Scheme")) {
	    int returnVal = fc.showOpenDialog(table);
	    if (returnVal == JFileChooser.APPROVE_OPTION) {
		File file = fc.getSelectedFile();
		fc.setCurrentDirectory(file);
		am.annotationfile = file;
		mtm.setUpTable(table);
		ml.setUpList(list);
	    }

	} else if(sourceText.equals("As Table")) {
	    setUpTable();

	} else if(sourceText.equals("As List")) {
	    setUpList();

	} else if(sourceText.equals("About")) {
	    JOptionPane.showMessageDialog(table, "This tool was created in the NLP lab at Stony Brook University.");
	}
	
	String caption = model_type + " " + model_compare + " ";
	caption += am.toString();
	frame.setTitle(caption);
    }

    private JMenuBar setUpMenus() {
	JMenuBar menuBar;
	JMenu menu;
	JMenuItem menuItem;

	menuBar = new JMenuBar();

	menu = new JMenu("File");
	menuBar.add(menu);

	menuItem = new JMenuItem("Open File");
	menuItem.addActionListener(this);
	menu.add(menuItem);
	menuItem = new JMenuItem("Open Second File");
	menuItem.addActionListener(this);
	menu.add(menuItem);
	menuItem = new JMenuItem("Save As");
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menu = new JMenu("Annotations");
	menuBar.add(menu);

	menuItem = new JMenuItem("Open Annotation Scheme");
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menu = new JMenu("Interface");
	menuBar.add(menu);

	menuItem = new JMenuItem("As List");
	menuItem.addActionListener(this);
	menu.add(menuItem);
	menuItem = new JMenuItem("As Table");
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menu = new JMenu("Other");
	menuBar.add(menu);
	
	menuItem = new JMenuItem("About");
	menuItem.addActionListener(this);
	menu.add(menuItem);

	return menuBar;
    }

    private static void createAndShowGUI() {
        JFrame.setDefaultLookAndFeelDecorated(true);

        frame = new JFrame("UtteranceTableModel");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        AnnotationPanel newContentPane = new AnnotationPanel();
        newContentPane.setOpaque(true); //content panes must be opaque
        frame.setContentPane(newContentPane);
        frame.setJMenuBar(newContentPane.setUpMenus());

        frame.pack();
        frame.setVisible(true);
    }

    public static void main(String[] args) {
    	
    	if(args.length <=0)
    	{
    		System.out.println("Usage: AnnotationPanel {comm|anno} [schema file] [{compare}]");
    	}
    	
    	model_type = args[0];
    	if(args.length >1)
    		model_file = args[1];
    	if(args.length >2)
    		model_compare = args[2];
    	
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        });
    }
}
