import java.util.Vector;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;


/***************************************************
 * 
 * @author sveta
 *
 * specific methods for Monroe annotation model
 */
class CommAnnotationModel extends AnnotationModel {
	
	protected NodeList getUtterances()
	{
		return doc.getElementsByTagName("utterance");
	}
	
	protected NodeList getUtterances2()
	{
		return doc2.getElementsByTagName("utterance");
	}	
	/**********************************************
	 * 
	 * @param columnNames
	 * @return adds to columnNames vector additional values
	 *  	 
	 * */
	
    protected Vector getAttColumnNames(Vector columnNames){		
 	   NamedNodeMap docNodes = getUtterances().item(0).getAttributes();

 	   columnNames.add(new NamePair("number", 5, false));
 	   columnNames.add(new NamePair("speaker", 10, false ));
 	   columnNames.add(new NamePair("Utterance", 80, false, "text"));
 	   columnNames.add(new NamePair("ASR", 80, false, "text"));
 	   columnNames.add(new NamePair("COMMENT", 10, true, "text"));
 	    return columnNames;
 	    };	

	/***********************************************
	 *  
	 * @param node
	 * @return text to display in the Utterance column
	 * this object should be turn. 
	 * get the child of turn: <text type="transcription">
	 * and get its first child
	 */
	protected Object getUtteranceText(Node node)
	{
		NodeList allChildren = node.getChildNodes();
		//get all children
		try{
		for(int i = 0; i< allChildren.getLength(); i++ )
		{
			if(allChildren.item(i).getNodeName().equals("text") )
			{
			Element element = (Element )allChildren.item(i);
			//either system or transcription
			if(element.getAttributeNode("type").getValue().equals("transcription") 
					|| element.getAttributeNode("type").getValue().equals("system")) 
				{
					if(element.getFirstChild() != null )
						return element.getFirstChild().getNodeValue();
					else
						return "";
				}
			}
		}		
		}
		catch(Exception ex)
		{
		    return null;
		}
		return null;
	}
	
	/***********************************************
	 *  
	 * @param node
	 * @return if this is  a user turn, return asr recognition, else empty 
	 * get the child of turn: <text type="asr">
	 * and get its first child
	 */
	protected Object getASRText(Node node)
	{
		NodeList allChildren = node.getChildNodes();
		//get all children
		try{
		for(int i = 0; i< allChildren.getLength(); i++ )
		{
			if(allChildren.item(i).getNodeName().equals("text") )
			{
			Element element = (Element )allChildren.item(i);
			//either system or transcription
			if(element.getAttributeNode("type").getValue().equals("asr") )
				{
					if(element.getFirstChild() !=null )
						return element.getFirstChild().getNodeValue();
					else
						return "";
				}
			}
		}		
		}
		catch(Exception ex)
		{
		    return null;
		}
		return "";
	}	
	
	/************************************************
	 * 
	 * @param colName - column name
	 * @param thisnode - xml node to retrieve value from
	 * @return if the column name is "Utterance" return utterance text, else 
	 * 			return the the attribute value using col name
	 */	
	protected Object getValueAt(String colName, Node thisnode)
	{
		if(colName.equals("Utterance"))
		{
			return getUtteranceText(thisnode);
		}
		else
			if(colName.equals("ASR"))
			{
				return getASRText(thisnode);
			}		
		else
		if(colName.equals("number"))
		{
			return thisnode.getAttributes().getNamedItem(colName).getNodeValue();
		}		
		else
		//user utterances and some of the speaker utterances do not have this 
		//attribute set. Go one level up and get it from the turn node
		if(colName.equals("speaker"))
		{
			Node parent = thisnode.getParentNode();
			return parent.getAttributes().getNamedItem(colName).getNodeValue();
		}		
		else
		//TODO this code is annotation specific. It should not be here.
		//this is the annotation tag
		//if an utterance node has a child dialog_act with speech_act="explicit_confirm"
		// or implicit_confirm, set the value of this cell to impl or expl
		// also import 	
		if(colName.equals("SystemImported"))
		{
			NodeList children = thisnode.getChildNodes();
			String sysimp = "";
			for(int i = 0; i<children.getLength(); i++)			
			{
				
				if(children.item(i).getNodeName().equals("dialogue_act") )
				{
					Element element = (Element )children.item(i);	
					
					if (element.getAttribute("speech_act")!=null )
					{
						sysimp += ", " + element.getAttribute("speech_act");
					}
					
					if(	element.getAttribute("task_subtask")!=null)
					{
						sysimp += ", " + element.getAttribute("task_subtask");
					}
					
					/*	if(	element.getAttribute("speech_act").equals("explicit_confirm"))
						{
							sysimp += "expl,";						
						}						
						if(	element.getAttribute("speech_act").equals("implicit_confirm"))
						{
							sysimp += "impl,";						
						}
						if(	element.getAttribute("task_subtask").equals("meta_start_over"))
						{
							sysimp +=  "startover,";						
						}
						if(	element.getAttribute("task_subtask").equals("meta_slu_reject"))
						{
							sysimp += "reject,";						
						}		
						if(	element.getAttribute("task_subtask").equals("meta_instruct"))
						{
							sysimp += "aid";						
						}				
						*/
				}
			}
			return sysimp;
			

			
			
			}
		else
		{
			//last resort: match the attribute name to the col name
			if(thisnode.getAttributes().getNamedItem(colName) != null)
			{
				//get column value and comment
				String val =  thisnode.getAttributes().getNamedItem(colName).getNodeValue();
				//get the comment
				if(thisnode.getAttributes().getNamedItem(colName+"_comment")!=null)
				    {
						return val+"+"+thisnode.getAttributes().getNamedItem(colName+"_comment").getNodeValue();
				    }
				else
					return val;
			}
			else
				return "";

		}
 	}	
}