import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;

import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/******************************************************
 * 
 * @author svetlana
 * 
 * Utterance table model displays columns: 
 * first all columns of file1 
 * then all columns of file 2
 *  utterance, asr, 
 *
 * 
 * Utterance table model
 * First  columns of the table are the defined by the annotation schema file
 * 
 * Next, the columns are specific to the annotation scheme of the input
 * 
 * The last column is always 'Utterance'
 *  
 */

class CompareTableModel extends UtteranceTableModel {


    public CompareTableModel(AnnotationModel a) {
    	super(a);
    }

    //2 files
    protected void setUpTable(JTable table) {
	if(am.annotationfile != null) {
	    am.schema = am.readDocument(am.annotationfile);
	}
	if(am.infile != null) {
	    am.doc = am.readDocument(am.infile);
	}
	if(am.infile2 != null) {
	    am.doc2 = am.readDocument(am.infile2);
	}	
	populateNames();
	fireTableStructureChanged();
	setUpAnnotationColumns(table);
	setColumnWidths(table);
    }



    protected void setUpAnnotationColumns(JTable table) {    	
        for(int index = 0; (index < getColumnCount()) && (((NamePair)columnNames.elementAt(index)).editable == true); index++) {
	    JComboBox comboBox = new JComboBox(am.getAnnotationLabels(index));
	    table.getColumnModel().getColumn(index).setCellEditor(new DefaultCellEditor(comboBox));
	}
    //make the last 2 columnts roll over.
    // TODO. this is table specific
        
        
	table.getColumnModel().getColumn(getColumnCount()-1).setCellRenderer(new TextAreaRenderer());
	table.getColumnModel().getColumn(getColumnCount()-2).setCellRenderer(new TextAreaRenderer());
	table.getColumnModel().getColumn(getColumnCount()-3).setCellRenderer(new TextAreaRenderer());

	fireTableChanged(new TableModelEvent(table.getModel()));
    }
	
    /****************************************
     *  populateNames creates table names 
     */
    protected void populateNames() {
	columnNames = new Vector();
	
	//load information from the schema
	if(am.schema != null) {
	    NodeList nodes = am.schema.getElementsByTagName("category");
	    for(int i = 0; i < nodes.getLength(); i++) {
			Element element = (Element )nodes.item(i);

			//by default attributes are modifiable, 
			//but if the modify attribute is false, then they are not
			boolean modify = true;
			//TODO: due to a bug this does not work
			//if(element.getAttributeNode("modify")!=null &&
			//		element.getAttributeNode("modify").getValue().equals("false")) 
			//			modify = false;
			String colname = element.getAttributes().getNamedItem("name").getNodeValue();
			
			columnNames.add(new NamePair("a1"+colname, 100, modify));
	    }
	    
	    for(int i = 0; i < nodes.getLength(); i++) {
			Element element = (Element )nodes.item(i);

			//by default attributes are modifiable, 
			//but if the modify attribute is false, then they are not
			boolean modify = true;
			//TODO: due to a bug this does not work
			//if(element.getAttributeNode("modify")!=null &&
			//		element.getAttributeNode("modify").getValue().equals("false")) 
			//			modify = false;
			String colname = element.getAttributes().getNamedItem("name").getNodeValue();
			
			columnNames.add(new NamePair("a2"+colname, 100, modify));
	    }
	    
	}

	//load table names from the document
	if(am.doc != null) {

		//get column names associated with attributes
	    columnNames = am.getAttColumnNames(columnNames);
	}
    }

    
    /*************************
     *  finds a value for given row and col
     *  remove a1 or a2 from the front of the string of col name
     */ 
    public Object getValueAt(int row, int col) {

    	NodeList nodes = null ;
    	Node thisnode=null;
    	
    	//first try to find if any of the column names correspond to an attribute
    	//else call annotation specific method
    	String colname = getColumnName(col);
    	if(colname.startsWith("a1") )
    	{
    		nodes = am.getUtterances();
    		colname = colname.substring(2);
        	//node of given row
        	thisnode = nodes.item(row);

    	}
    	else
    	if(colname.startsWith("a2") )
    	{
    		nodes = am.getUtterances2();
    		colname = colname.substring(2);
        	//node of given row
        	thisnode = nodes.item(row);

    	}   	
    	else
    	{
    		nodes = am.getUtterances();
         	//node of given row
        	thisnode = nodes.item(row);  	
    	}
    	
    	try{
    		return am.getValueAt(colname, thisnode);
	    } catch(Exception e) {
	    	//attribute for this column name does not exist
	    	return "";
		}     		
    }
    
}