import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;

import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/******************************************************
 * 
 * @author amanda
 * 
 * Utterance table model displays columns: 
 * 		utterance id	incremented automatically
 * 		speaker id		user/system
 * 		Utterance  		speech
 * 
 * Utterance table model
 * First  columns of the table are the defined by the annotation schema file
 * 
 * Next, the columns are specific to the annotation scheme of the input
 * 
 * The last column is always 'Utterance'
 *  
 */

class UtteranceTableModel extends AbstractTableModel {
    protected AnnotationModel am;
    protected Vector columnNames;

    public UtteranceTableModel(AnnotationModel a) { am = a; }

    protected void setUpTable(JTable table) {
	if(am.annotationfile != null) {
	    am.schema = am.readDocument(am.annotationfile);
	}
	if(am.infile != null) {
	    am.doc = am.readDocument(am.infile);
	}
	populateNames();
	fireTableStructureChanged();
	setUpAnnotationColumns(table);
	setColumnWidths(table);
    }

    protected void setColumnWidths(JTable table) {
	for(int i = 0; i < getColumnCount(); i++) {
	    TableColumn column = table.getColumnModel().getColumn(i);
	    column.setPreferredWidth(getColumnWidth(i));
	}
    }

    protected void setUpAnnotationColumns(JTable table) {    	

        for(int index = 0; index < getColumnCount(); index++) {
    	    JComboBox comboBox = new JComboBox(am.getAnnotationLabels(index));
    	    if (((NamePair)columnNames.elementAt(index)).editable == true)
    	    	table.getColumnModel().getColumn(index).setCellEditor(new DefaultCellEditor(comboBox));
    	    else
    	    	table.getColumnModel().getColumn(index).setCellRenderer(new TextAreaRenderer());
    	    	
    	}

	fireTableChanged(new TableModelEvent(table.getModel()));
    }
	
    /****************************************
     *  populateNames creates table names 
     */
    protected void populateNames() {
	columnNames = new Vector();
	
	//initial index column
	//columnNames.add(new NamePair("index", 5, false));
	
	//load information from the schema
	if(am.schema != null) {
	    NodeList nodes = am.schema.getElementsByTagName("category");
	    for(int i = 0; i < nodes.getLength(); i++) {
			Element element = (Element )nodes.item(i);

			//by default attributes are modifiable, 
			//but if the modify attribute is false, then they are not
			boolean modify = true;
			//TODO: due to a bug this does not work
			if(element.getAttributeNode("modify")!=null &&
					element.getAttributeNode("modify").getValue().equals("false"))
			{
						modify = false;
			}
			
			String width = element.getAttributeNode("width")!=null ? element.getAttributeNode("width").getValue() : "50";
			columnNames.add(new NamePair(element.getAttributes().getNamedItem("name").getNodeValue(), 
						new Integer(width).intValue(), modify));
	    }
	}

	//load table names from the document
	if(am.doc != null) {

		//get column names associated with attributes
	    columnNames = am.getAttColumnNames(columnNames);
	}
    }

    public int getColumnCount() {
	if(columnNames != null)
	    return columnNames.size();
	return 0;
    }
    
    public int getRowCount() {
	if(am.doc != null) {
	    return am.getUtterances().getLength();
	}
	return 0;
    }

    public int getColumnWidth(int col) {
	return ((NamePair)columnNames.elementAt(col)).width;
    }

    public String getColumnName(int col) {
	return ((NamePair)columnNames.elementAt(col)).name;
    }

    /*************************
     *  finds a value for given row and col
     */ 
    public Object getValueAt(int row, int col) {
    	
    	NodeList nodes = am.getUtterances();
    	//node of given row
    	Node thisnode = nodes.item(row);
    	
    	
		if(getColumnName(col).equals("index"))
		{
			return new Integer(row+1);
		}    	
    	//first try to find if any of the column names correspond to an attribute
    	//else call annotation specific method
    	try{
    		return am.getValueAt(getColumnName(col), thisnode);
	    } catch(Exception e) {
	    	//attribute for this column name does not exist
	    	return "";
		}     		
    }

    public Class getColumnClass(int c) {
	return getValueAt(0, c).getClass();
    }
    
    public boolean isCellEditable(int row, int col) {
	return ((NamePair)columnNames.elementAt(col)).editable;
    }
    
    public void setValueAt(Object value, int row, int col) {
	NodeList nodes = am.getUtterances();
	am.setAnnotation((Element)nodes.item(row), getColumnName(col), value);
    }
    

}